<?php


namespace App\Services\pay;

use Api\PhpServices\Idgen\Idgen;
use Api\PhpUtils\Log\FileLog;
use App\Models\Dictionary;
use App\Models\order\mysql\PayOrder;
use App\Models\order\mysql\PayOrderItem;
use App\Models\order\mysql\PayOrderClearing;
use App\Models\order\mysql\PayOrderClearingItem;
use App\Models\order\mysql\RefundOrder;
use App\Services\order\OrderService;

use App\Services\pingxx\PingxxService;
use App\Services\wallet\WalletService;
use Helpers\Strategy;
use Helpers\Sdk;

use App\Exception\custom\PayException;

class PayService
{
    // 订单信息
    private $order_info = [];
    // 子订单信息
    private $order_items = [];

    private $pay_order = [];
    private $clear_list = [];
    private $clear_items_list = [];

    public function do_pay($order_id, $user_id)
    {
        // 获取订单信息
        $data = OrderService::getFullOrderData($order_id, $user_id);
        if (empty($data['result']['order_info'])) {
            throw new PayException(['cus' => 1]);
        }

        $order_info = $data['result']['order_info'];
        if ($order_info['user_id'] != $user_id) {
            throw new PayException(['cus' => 1]);
        }

        $metadata = [
            'order_id'=>$order_id,
            'goods_sku_id'=>[],
            'order_item_id'=>[],
        ];
        foreach ($data['result']['order_item_list'] as $item) {
            array_push($metadata['goods_sku_id'], $item['goods_sku_id']);
            array_push($metadata['order_item_id'], $item['order_item_id']);
        }

        // 判断是否存在有效订单
        $max_expire_time = config('pay','pay.expire_time') ?? 1800;
        $expire_time = time() + $max_expire_time;
        $pay_order_id = $this->gen_pay_order_id($user_id);
        $new_pay_order = [
            'pay_order_id' => $pay_order_id,
            'user_id' => $order_info['user_id'],
            'life_account_id' => $order_info['life_account_id'],
            'shop_id' => $order_info['shop_id'],
            'order_id' => $order_info['order_id'],
            'pay_order_status' => Dictionary::O_PAY_STATUS_UNPAY,
            'pay_amount' => $order_info['payment'],
            'third_order_id' => '',
            'expire_time' => date('Y-m-d H:i:s', $expire_time),
            'source_name' => 10,
            'service_name' => 1,
            'extra' => json_encode([]),
        ];

        //获取或创建支付订单
        $pay_order = PayOrder::get_valid_order($order_id, $new_pay_order);
        if (empty($pay_order)) {
            throw new PayException(['cus' => 0]);
        }

        if ($pay_order['pay_order_status'] == Dictionary::O_PAY_STATUS_PAYED) {
            throw new PayException(['cus' => 3]);
        }

        // 支持幂等，存在有效订单需要可以重复支付
        $ret = PingxxService::getInstance()->createOrder($pay_order, $metadata);
        if (!empty($ret['error'])) {
            throw new \Exception($ret['error']['message'], '2001');
        }

        if ($ret["status"] == 'paid' || $ret['status'] == 'refunded') {
            $edit = [
                'pay_order_status' => Dictionary::O_PAY_STATUS_PAYED,
                'third_order_id' => $ret['id'],
            ];
            PayOrder::update($edit, ['pay_order_id' => $pay_order['pay_order_id']]);
            throw new PayException(['cus' => 3]);

        } elseif ($ret["status"] == 'canceled') {
            $edit = [
                'expire_time' => date('Y-m-d H:i:s'),
            ];
            PayOrder::update($edit, ['pay_order_id' => $pay_order['pay_order_id']]);
            throw new PayException(['cus' => 11]);

        } elseif ($ret["status"] == 'created') {
            $edit = [
                'pay_order_status' => Dictionary::O_PAY_STATUS_WAIT,
                'third_order_id' => $ret['id'],
            ];
            PayOrder::update($edit, ['pay_order_id' => $pay_order['pay_order_id']]);

        } else {
            throw new PayException(['cus' => 0]);
        }

        //存在一个继续支付单订单
        if($pay_order_id != $pay_order['pay_order_id']) {
            $ret['exist'] = true;
        }

        // 创建支付单
        $payment = [
            'charge_order_no'=> $pay_order['pay_order_id'],
            'charge_amount'=> $pay_order['pay_amount'],
            'channel'=> 'wx',
        ];

        $pay = PingxxService::getInstance()->pay($ret['id'], $payment);
        if (!empty($pay['error'])) {
            throw new \Exception($pay['error']['message'], '2001');
        }

        return $pay;
    }

    /**
     * @throws PayException
     * 回调订单支付
     */
    public function call_back($data)
    {
        if ($data['type'] != 'order.succeeded') {
            throw new PayException(['cus' => 7]);
        }

        if (empty($data['data']['object'])) {
            throw new PayException(['cus' => 6]);
        }

        $object = $data['data']['object'];
        /*
        $ping_data = PingxxService::getInstance()->getOrder($object['id']);
        if($ping_data['status'] != 'paid') {
            throw new PayException(['cus' => 5]);
        } //actual_amount 金额核对
        */
        // 修改状态，锁定业务
        $pay_order_id = $object['merchant_order_no'];
        $edit = [
            'pay_order_status' => Dictionary::O_PAY_STATUS_PAYED,
            'pingxx_callback_success_time' => date('Y-m-d H:i:s'),
        ];
        $where = [
            'pay_order_id' => $pay_order_id,
            'pay_order_status' => Dictionary::O_PAY_STATUS_WAIT
        ];
        $cnt = PayOrder::update($edit, $where);
        $pay_order = PayOrder::getMaster('*', ['pay_order_id' => $pay_order_id]);
        if (empty($pay_order)) {
            throw new PayException(['cus' => 5]);
        }

        //保持幂等返回成功
        if($cnt == 0 && $pay_order['pay_order_status'] == Dictionary::O_PAY_STATUS_PAYED) {
            return ['pay_order_id' => $pay_order_id];
        }

        if($pay_order['pay_amount'] != $object['actual_amount']) {
            throw new PayException(['cus' => 15]);
        }

        //获取订单 + 子单 + 分销信息
        $ret = OrderService::getFullOrderData($pay_order['order_id'], $pay_order['user_id']);
        if (empty($ret['result']['order_info'])) {
            throw new PayException(['cus' => 1]);
        }

        $this->pay_order = $pay_order;

        // 生成支付清分单
        $this->make_order_clearing($ret['result']);

        return ['pay_order_id' => $pay_order_id];
    }

    /**
     * @param $order_item_id
     * @param $life_account_id
     * @throws \App\Exception\custom\CodeSpecialException
     * 子订单核销
     */
    public function write_off($order_item_id, $life_account_id)
    {
        //锁定状态
        try{
            PayOrderItem::beginTransaction();
            $cnt = PayOrderItem::update([
                'notify_account_status' => 0,
                'can_notify_account' => 1,
                'can_notify_account_time' => date('Y-m-d H:i:s'),
            ], ['order_item_id' => $order_item_id, 'can_notify_account'=>0]);

            $refund = RefundOrder::getMaster('*', ['order_item_id'=>$order_item_id]);
            if(!empty($refund)) {
                throw new PayException(['cus'=>8]);
            }

            $item = PayOrderItem::getMaster('*', ['order_item_id' => $order_item_id]);
            if(empty($item)) {
                throw new PayException(['cus'=>9]);
            }

            $pay_order = PayOrder::getMaster('*', ['pay_order_id'=>$item['pay_order_id']]);
            if(empty($pay_order) || $pay_order['life_account_id'] != $life_account_id) {
                throw new PayException(['cus'=>10]);
            }

            PayOrderItem::commit();
        }catch (\Exception $e) {
            PayOrderItem::rollback();
            throw $e;
        }

        if($cnt == 0) {//幂等直接返回
            return ['order_item_id' => $order_item_id];
        }

        $this->sync_account_wallet($order_item_id);

        return ['order_item_id' => $order_item_id];
    }

    /**
     * @param $order_item_id
     * @throws \App\Exception\custom\CodeSpecialException
     * 同步钱包处理
     */
    public function sync_account_wallet($order_item_id) {

        $cnt = PayOrderItem::update([
            'notify_account_status' => 1,
        ], ['order_item_id' => $order_item_id, 'can_notify_account' => 1, 'notify_account_status'=>0]);

        if($cnt ==0) {//乐观锁，不满足条件不处理
            return true;
        }

        $clearing_items = PayOrderClearingItem::select('*',
            ['order_item_id' => $order_item_id]
        );

        $wallet_list = [];
        foreach ($clearing_items as $row) {
            if($row['pay_amount'] <= 0) {
                continue;
            }

            $wallet_list[] = [
                'user_id' => $row['account_id'],
                'service_name' => 10,
                'source_name' => 1,
                'amount' => $row['pay_amount'],
                'third_order_id' => $row['pay_order_clearing_item_id'],
                'third_order_id_type' => $row['pay_sub_type'],
                'third_order_desc' => $row['remark'],
            ];
        }

        try{
            if($wallet_list) { //同步成功，返回结果
                WalletService::send(['list'=>$wallet_list]);
            }
            PayOrderItem::update([
                'notify_account_status' => 2,
                'notify_account_success_time' => date('Y-m-d H:i:s')
            ], ['order_item_id' => $order_item_id, 'can_notify_account' => 1]);

        } catch (\Exception $e) {
            FileLog::error('wallet: 钱包同步失败', json_encode($wallet_list, JSON_UNESCAPED_UNICODE), $e);
            //补偿处理
            PayOrderItem::update([
                'notify_account_status' => 0,
            ], ['order_item_id' => $order_item_id, 'can_notify_account' => 1]);
        }
    }

    /**
     * @param $data
     * 构造订单流水
     */
    private function make_order_clearing($data)
    {
        $this->order_info = $data['order_info'];
        $this->order_items = $data['order_item_list'];

        $this->clear_list = [];
        $this->clear_items_list = [];

        $pay_order_items = [];
        foreach ($this->order_items as $r) {
            $pay_order_items[] = [
                'pay_order_item_id' => $this->gen_pay_order_item_id(),
                'pay_order_id' => $this->pay_order['pay_order_id'],
                'order_id' => $this->pay_order['order_id'],
                'order_item_id' => $r['order_item_id'],
                'user_id' => $this->pay_order['user_id'],
            ];
        }

        $wx_tip = $this->make_tip_clearing(); //微信手续费
        $distribution_tip = $this->make_distribution_clearing(); //团长收益
        $platform_tip = $this->make_platform_clearing(); //平台收益
        $merchant_cash = $this->order_info['payment'] - $wx_tip - $distribution_tip - $platform_tip;

        //核对各项资金的合理性
        if($wx_tip <0 || $distribution_tip <0 || $platform_tip < 0 || $merchant_cash < 0) {
            FileLog::error('pay: 支付预结算失败，结算资金为负',
                json_encode([$this->order_info['payment'], $wx_tip, $distribution_tip, $platform_tip, $merchant_cash]));
            throw new PayException(['cus'=>14]);
        }

        $this->make_merchant_clearing($merchant_cash); //商户收益

        try {
            PayOrderClearing::beginTransaction();
            $c_cnt = PayOrderItem::insert($pay_order_items, ['rowCount' => true]);
            $i_cnt = PayOrderClearing::insert($this->clear_list, ['rowCount' => true]);
            $ci_cnt = PayOrderClearingItem::insert($this->clear_items_list, ['rowCount' => true]);

            if ($i_cnt * $c_cnt * $ci_cnt == 0) {
                throw new PayException(['cus' => 0]);
            }

            PayOrderClearing::commit();
        } catch (\Exception $e) {
            PayOrderClearing::rollback();
            throw new PayException(['cut' => 5]);
        }

        //如果是不可退款商品，则需进入结算队列
        foreach ($this->order_items as $r) {
            // 收集直接到账信息
            if($r['rule_refund'] == 1 ) { //分销商品一定是不可退
                PayOrderItem::update([
                    'can_notify_account'=>1,
                    'can_notify_account_time'=>date('Y-m-d H:i:s')
                ], ['order_item_id'=>$r['order_item_id']]);
            }
        }
    }

    /**
     * @param $payment
     * @return false|float
     * 获取微信手续费
     */
    private function make_tip_clearing()
    {
        $wx_rate = config('pay','pay.wechat_rate') ?? 60; //微信利率 60 / 10000
        $wx_account_id = config('pay','pay.wechat_account_id') ?? '1111';

        $total_tip = round($wx_rate * $this->order_info['payment'] / 10000);
        if (empty($total_tip)) {
            return 0;
        }

        $account = [
            'account_id' => $wx_account_id,
            'amount' => $total_tip,
        ];

        $account['account_type'] = 4; //三方手续费 + 平台总帐
        $account['pay_type'] = 2; //支付
        $account['pay_sub_type'] = 406; //第三方渠道手续费
        $account['need_recorded'] = 0; //不需同步给平台

        /* 微信手续费总单，在退款、核销的时候会更新这个金额，保证他的准确 */
        $this->do_clearing_data($account, $total_tip, true);
        return $total_tip;
    }

    /**
     * @param $total
     * @return false|float
     * 计算平台收入
     */
    private function make_platform_clearing()
    {
        $account = [
            'account_id' => '100020003201',
            'amount' => 0,
            'rate' => 3000,
            'title' => '平台收益',
        ];

        $total_tip = $account['amount'] = round($this->order_info['total_price'] * $account['rate'] / 10000);
        if (empty($total_tip)) {
            return 0;
        }

        $account['account_type'] = Dictionary::ACCOUNT_TYPE_SS;
        $account['pay_type'] = Dictionary::PAY_TYPE_IN;
        $account['pay_sub_type'] = Dictionary::PAY_SUB_TYPE_SYS_IN;
        $account['need_recorded'] = Dictionary::NO;

        $this->do_clearing_data($account, $total_tip, false);
        return $total_tip;
    }

    /**
     * @return mixed
     * 计算分销信息, 分销没有总帐，仅有单条floor的计算
     */
    private function make_distribution_clearing()
    {

        /**
         * 'distributor_user_id'=>122234,
         * 'distributor_commission_value'=>60,
         * 'parent_user_id'=>122234 . rand(111, 999),
         * 'parent_commission_value'=>60,
         */

        $total_tip = 0;
        foreach ($this->order_items as $r) {
            //仅处理分销业务
            if(empty($r['distributor_user_id']) || $r['marketing_type'] != 1) {
                continue;
            }

            $_one = $_two = 0;
            if($r['commission_mode'] == 1) {
                $_one = floor($r['distributor_commission_value'] * $this->order_info['total_price'] / 10000);
                $_two = floor($r['parent_commission_value'] * $this->order_info['total_price'] / 10000);

            } elseif($r['commission_mode'] == 2) {
                $_one = $r['distributor_commission_value'];
                $_two = $r['parent_commission_value'];
            }

            $users[0] = [
                'user_id'=>$r['distributor_user_id'],
                'value'=>$r['distributor_commission_value'],
                'amount'=>$_one
            ];

            if(!empty($r['parent_user_id'])) {
                $users[1] = [
                    'user_id'=>$r['parent_user_id'],
                    'value'=>$r['parent_commission_value'],
                    'amount'=>$_two
                ];
            }

            foreach ($users as $u) {
                if (empty($r['user_id']) || $u['amount'] == 0) {
                    continue;
                }

                $this->clear_items_list[] = [
                    'pay_order_clearing_item_id' => $this->gen_pay_order_clearing_item_id(),
                    'pay_order_clearing_id' => '',
                    'pay_order_id' => $this->pay_order['pay_order_id'],
                    'order_id' => $this->pay_order['order_id'],
                    'order_item_id' => $r['order_item_id'],
                    'account_id' => $u['user_id'],
                    'account_type' => 1, //用户
                    'pay_amount' => $u['amount'],
                    'pay_type' => 1, //收入
                    'pay_sub_type' => 105, //团长佣金收入
                    'need_recorded' => 1,
                ];

                $total_tip += $u['amount'];
            }
        }

        return $total_tip;
    }

    /**
     * @param $merchant_cash
     * @return mixed
     * 计算商户清分信息
     */
    private function make_merchant_clearing($merchant_cash)
    {
        $ret = $this->get_life_account($this->order_info['life_account_id']);
        if(empty($ret['result']['life_account_admin_id'])) {
            $log_string = json_encode([
                'life_account_id'=>$this->order_info['life_account_id'],
                'ret'=>$ret,
            ], JSON_UNESCAPED_UNICODE);

            FileLog::error('pay:', '获取生活号管理员失败', $log_string);
            throw new PayException(['cus'=>13]);
        }

        $account = [
            'account_id' => $ret['result']['life_account_admin_id'],
            'amount' => $merchant_cash,
            'title' => '商家订单收益',
        ];

        $account['account_type'] = 2; //商户
        $account['pay_type'] = 1; //收入
        $account['pay_sub_type'] = 102; //商品交易
        $account['need_recorded'] = 1; //同步到平台总帐

        $this->do_clearing_data($account, $merchant_cash, false);
        return $merchant_cash;
    }

    private function do_clearing_data($account, $total_tip, $is_wx = false)
    {
        $total_amount = $this->order_info['payment']; //订单支付总价格
        $cleared_amount = 0; //已经结算记录
        $cleared_tip = 0; //已经结算记录

        $pay_order_clearing_id = $this->gen_pay_order_clearing_id();
        $this->clear_list[] = [
            'pay_order_clearing_id' => $pay_order_clearing_id,
            'pay_order_id' => $this->pay_order['pay_order_id'],
            'order_id' => $this->pay_order['order_id'],
            'account_id' => $account['account_id'],
            'account_type' => $account['account_type'],
            'pay_amount' => $account['amount'],
            'pay_type' => $account['pay_type'],
            'pay_sub_type' => $account['pay_sub_type'],
            'need_recorded' => $account['need_recorded'],
        ];

        if ($is_wx) { //微信不计算手续费拆弹
            return true;
        }

        foreach ($this->order_items as $r) {

            $current_tip = Strategy::getTip($r['payment'], $total_amount, $cleared_amount, $total_tip, $cleared_tip);
            $id = $this->gen_pay_order_clearing_item_id();


            //商家资金需要显示商品名称
            if($account['pay_sub_type'] == 102) {
                $remark = $r['goods_name'] ?? '';
                if(mb_strlen($remark) > 100) {
                    $remark = mb_substr($remark, 0, 100) . '...';
                }
            } else {
                $remark = '';
            }
            $this->clear_items_list[] = [
                'pay_order_clearing_item_id' => $id,
                'pay_order_clearing_id' => $pay_order_clearing_id,
                'pay_order_id' => $this->pay_order['pay_order_id'],
                'order_id' => $this->pay_order['order_id'],
                'order_item_id' => $r['order_item_id'],
                'account_id' => $account['account_id'],
                'account_type' => $account['account_type'],
                'pay_amount' => $current_tip,
                'pay_type' => $account['pay_type'],
                'pay_sub_type' => $account['pay_sub_type'],
                'remark' => $remark,
                'need_recorded' => $account['need_recorded'],
            ];

            $cleared_amount += $r['payment'];
            $cleared_tip += $current_tip;
        }
    }

    private function gen_pay_order_id($user_id)
    {
        $number = substr($user_id, -2);
        $ids = $this->get_idgen_id($number, 1);
        return array_pop($ids);
    }

    private function gen_pay_order_item_id()
    {
        $number = substr($this->order_info['user_id'], -2);
        $ids = $this->get_idgen_id($number, 1);
        return array_pop($ids);
    }

    private function gen_pay_order_clearing_id()
    {
        $number = substr($this->order_info['user_id'], -2);
        $ids = $this->get_idgen_id($number, 1);
        return array_pop($ids);
    }

    private function gen_pay_order_clearing_item_id()
    {
        $number = substr($this->order_info['user_id'], -2);
        $ids = $this->get_idgen_id($number, 1);
        return array_pop($ids);
    }

    private function get_idgen_id($number, $count = 1)
    {
        $number = intval($number);
        $res = Idgen::get(appConfig('idgen.partner'), appConfig('idgen.key'), [], [[
            "type" => "55c768",
            'number' => $number,
            "count" => $count]]);
        $id = $res['id_datetime']['55c768'] ?? [];
        return $id;
    }

    private function get_life_account($life_account_id)
    {
        $url = config('interface', 'merchant.lifeaccount.get_life_account_by_id');
        if (!$url) {
            throw new CodeSpecialException("failed" . __METHOD__);
        }
        $params = ['life_account_id' => $life_account_id];
        return Sdk::call($url, $params);
    }
}