<?php


namespace App\Services\refund;

use Api\PhpServices\Idgen\Idgen;
use App\Models\Dictionary;
use App\Models\order\mysql\PayOrder;
use App\Models\order\mysql\PayOrderClearing;
use App\Models\order\mysql\PayOrderClearingItem;
use App\Models\order\mysql\PayOrderItem;
use App\Models\order\mysql\RefundOrder;
use App\Services\order\OrderService;

use App\Services\pingxx\PingxxService;
use Helpers\Strategy;
use Helpers\Logger;

use App\Exception\custom\RefundException;

class RefundService
{

    private $refund_order = [];

    /**
     * @param $order_item_id
     * @param $user_id
     * @throws RefundException
     * 发起退款请求
     */
    public function do_refund($order_item_id, $user_id) {

        // 获取子单信息
        $ret = OrderService::getOrderItemData($order_item_id, $user_id);
        if(empty($ret['result'])) {
            throw new RefundException(['cus' => 1]);
        }

        if(empty($ret['result']['user_id'] != $user_id)) {
            throw new RefundException(['cus' => 1]);
        }

        $order_item = $ret['result'];
        $where = [
            'order_item_id'=>$order_item_id,
            'user_id'=>$user_id,
            'order_id'=>$order_item['order_id'],
            'refund_amount'=>$order_item['payment'],
            'refund_order_id'=>$this->gen_refund_order_id($user_id),
        ];

        $data = $this->can_refund($where, $order_item);

        // 构造退款数据
        $refund = [
            'charge'=>$data['pay_order']['third_order_id'],
            'charge_amount'=>$data['refund_order']['refund_amount'],
            'metadata' => [
                'refund_order_id'=>$data['refund_order']['refund_order_id'],
                'order_item_id'=>$data['refund_order']['order_item_id'],
                'goods_sku_id'=>$order_item['goods_sku_id'] ?? '',
            ],
        ];

        try{
            $ret = PingxxService::getInstance()->sendRefund($refund);
            if(!empty($ret["data"])) {
                $edit = ['request_pingxx_success_time'=>date('Y-m-d H:i:s')];
                RefundOrder::update($edit, ['refund_order_id'=>$data['refund_order']['refund_order_id']]);
            } else {
                throw new \Exception($ret['error']['message'], '2001');
            }

            return ['order_item_id'=>$order_item_id];

        }catch (\Exception $e) {
            $edit = ['refund_order_status'=>Dictionary::REFUND_ORDER_STATUS_UNDO,];
            RefundOrder::update($edit, ['refund_order_id'=>$data['refund_order']['refund_order_id']]);
            throw $e;
        }
    }

    public function call_back($data) {
        if(empty($data['data']['object'])) {
            throw new RefundException(['cus' => 6]);
        }

        $object = $data['data']['object'];
        if(empty($object['metadata']['refund_order_id'])) {
            throw new RefundException(['cus' => 0]);
        }

        /*
        // 获取退款回调信息，确认订单ID
        $charge = $object['charges']['data'][0];
        $ping_refund = PingxxService::getInstance()->getRefund($charge['id'], $object['id']);
        if($ping_refund['status'] != 'succeeded') {
            throw new RefundException(['cus' => 6]);
        }
        */

        // 修改状态，锁定业务
        $refund_order_id = $object['metadata']['refund_order_id'];
        $this->lock_refund_clear($refund_order_id);

        //不用搞清分核算，是否需要平台记账？
        $this->make_refund_clear($refund_order_id);
    }

    /**
     * @param $where
     * @return \Api\PhpUtils\Mysql\MysqlBase|array
     * @throws RefundException
     * 判断订单是否可退，判断是否存在退款申请
     */
    private function can_refund($where, $order_item) {

        /**
         * 业务方自行解决，订单关闭、订单超过时长不允许退款等逻辑
         * 支付中心退款判断仅做主单是否支付，主单是否支持退款的判断
         */

        try{
            RefundOrder::beginTransaction();
            $pay_order_item = PayOrderItem::getMaster('*', ['order_item_id'=>$where['order_item_id']]);
            if(empty($pay_order_item)) {
                throw new RefundException(['cus' => 4]);
            }
            if($pay_order_item['can_notify_account'] == 1) {
                throw new RefundException(['cus' => 7]);
            }

            $refund_order = RefundOrder::getMaster('*', ['order_item_id'=>$where['order_item_id']]);
            if(!empty($refund_order)) {
                if($refund_order['refund_order_status'] == Dictionary::REFUND_ORDER_STATUS_OK) {
                    throw new RefundException(['cus' => 2]);
                }

                if($refund_order['refund_order_status'] == Dictionary::REFUND_ORDER_STATUS_SEND) {
                    throw new RefundException(['cus' => 3]);
                }

            } else {
                $refund_order = $where;
                $refund_order['refund_order_status'] = Dictionary::REFUND_ORDER_STATUS_UNDO;
                //$refund_order['refund_amount'] = 0;
                $cnt = RefundOrder::insert($refund_order, ['rowCount'=>true]);
                if($cnt == 0) {
                    throw new RefundException(['cus' => 0]);
                }
            }

            //重新计算微信手续费
            $cleared_amount = $cleared_tip = 0;

            $pay_order = PayOrder::getMaster('*', ['order_id'=>$where['order_id']]);
            // 总共手续费
            $wx_tip = PayOrderClearing::getMaster('*', [
                'order_id'=>$where['order_id'],
                'pay_order_id'=>$pay_order['pay_order_id'],
                'pay_type'=>Dictionary::PAY_TYPE_IN]
            );

            // 计算已退金额、已退手续费
            $list = RefundOrder::selectMaster('*', ['order_id'=>$where['order_id']]);
            foreach ($list as $r) {
                if($r['refund_order_status'] == Dictionary::REFUND_ORDER_STATUS_UNDO) {
                    continue;
                }

                $cleared_amount += $r['refund_amount'];
                $cleared_tip += $r['refund_wx_tip'];
            }

            $wx_tip = Strategy::getWxTip($order_item['payment'], $pay_order['pay_amount'], $cleared_amount, $wx_tip['pay_amount'], $cleared_tip);

            $edit = [
                'refund_order_status'=>Dictionary::REFUND_ORDER_STATUS_SEND,
                'refund_amount'=>$order_item['payment'],
                'refund_wx_tip'=>$wx_tip,
            ];
            $ret = RefundOrder::update($edit, ['refund_order_id'=>$refund_order['refund_order_id']]);
            if(!$ret) {
                throw new RefundException(['cus' => 0]);
            }

            //重新获取
            $refund_order = RefundOrder::getMaster('*', ['refund_order_id'=>$refund_order['refund_order_id']]);
            $data = [
                'refund_order'=>$refund_order,
                'pay_order'=>$pay_order,
            ];

            RefundOrder::commit();

        }catch (\PDOException $e) {
            RefundOrder::rollback();
            throw $e;
        }

        return $data;
    }

    private function lock_refund_clear($refund_order_id) {

        try{
            RefundOrder::beginTransaction();
            $refund_order = RefundOrder::get('*', ['refund_order_id'=>$refund_order_id]);
            if(empty($refund_order)) {
                throw new RefundException(['cus' => 0]);
            }

            $this->refund_order = $refund_order;

            if($refund_order['refund_order_status'] == Dictionary::REFUND_ORDER_STATUS_OK) {
                throw new RefundException(['cus' => 0]);
            }

            $edit = [
                'refund_order_status'=>Dictionary::REFUND_ORDER_STATUS_OK,
                'pingxx_callback_success_time'=>date('Y-m-d H:i:s'),
                'notify_account_status'=>Dictionary::SEND_ACCOUNT_STATUS_DONE,
            ];

            RefundOrder::update($edit, ['refund_order_id'=>$refund_order_id]);
            RefundOrder::commit();

        }catch (\PDOException $e) {
            RefundOrder::rollback();

            throw $e;
        }
    }

    private function make_refund_clear() {
        return true;
        //仅处理WX手续费，平台补贴，商户补贴
    }

    private function gen_refund_order_id($user_id)
    {
        $number = substr($user_id, -2);
        $ids = $this->get_idgen_id($number, 1);
        return array_pop($ids);
    }

    private function get_idgen_id($number, $count = 1)
    {
        $number = intval($number);
        $res = Idgen::get(appConfig('idgen.partner'), appConfig('idgen.key'), [], [[
            "type" => "55c768",
            'number' => $number,
            "count" => $count]]);
        $id = $res['id_datetime']['55c768'] ?? [];
        return $id;
    }

}